#!/usr/bin/env python3
"""
Discussion Generation Module for Report 001

Generate LLM-powered discussions for virus surveillance visualizations.
Supports both local development and GitHub Actions environments.

This module creates AI-generated discussions for each city's virus surveillance
data, using the same LLM prompts and logic as generate_report_cards.py but
as a standalone, reusable component.
"""

import sys
import os
import shutil
import tomllib
from pathlib import Path
from typing import Dict, List, Any, Optional
import pandas as pd
import numpy as np
import json
from datetime import datetime
import requests
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn
import argparse
import pickle
import gzip
import re

# Load environment variables from .env file if running locally
try:
    from dotenv import load_dotenv
    # Look for .env file in the project root
    env_path = Path(__file__).parent.parent.parent / '.env'
    if env_path.exists():
        load_dotenv(env_path)
except ImportError:
    # dotenv not available, rely on environment variables being set externally
    pass

# Import our data modules
from create_report_card_data import create_report_card_data
from retrieve_virus_data import retrieve_all_virus_data
from sites_config import get_public_sites, get_private_sites

# Environment Detection
def detect_environment() -> str:
    """Detect if running locally or in GitHub Actions."""
    return "github-actions" if os.getenv('GITHUB_ACTIONS') else "local"


def get_output_base_dir(environment: str) -> Path:
    """Get base output directory based on environment."""
    script_dir = Path(__file__).parent
    if environment == "github-actions":
        return script_dir / "github-actions-outputs"
    else:
        return script_dir / "local-outputs"


def setup_output_directories(base_dir: Path) -> Dict[str, Path]:
    """Setup all required output directories."""
    directories = {
        'discussions_private': base_dir / 'discussions_private',
        'discussions_public': base_dir / 'discussions_public',
        'visualizations_public': base_dir / 'visualizations_public',
        'visualizations_private': base_dir / 'visualizations_private'
    }
    
    for dir_path in directories.values():
        dir_path.mkdir(parents=True, exist_ok=True)
    
    return directories


class AzureOpenAIConfig:
    """Azure OpenAI configuration with environment-aware loading."""
    
    def __init__(self):
        self.api_key = self._get_api_key()
        self.endpoint = os.getenv('AZURE_OPENAI_ENDPOINT')
        self.deployment = os.getenv('AZURE_OPENAI_DEPLOYMENT')
        self.api_version = os.getenv('AZURE_OPENAI_API_VERSION')
    
    def _get_api_key(self) -> str:
        """Get API key from environment (local .env or GitHub Secrets)."""
        return os.getenv('AZURE_OPENAI_API_KEY')
    
    def is_configured(self) -> bool:
        """Check if OpenAI is properly configured."""
        return bool(self.api_key)


# Cache for virus configuration to avoid repeated file loading
_virus_config_cache = None
_virus_category_cache = {}


def load_virus_config(config_path: Optional[Path] = None) -> Dict[str, Any]:
    """Load virus configuration from virus.toml file."""
    if config_path is None:
        config_path = Path(__file__).parent / "virus.toml"
    else:
        config_path = Path(config_path)
    
    if not config_path.exists():
        raise FileNotFoundError(f"Virus configuration file not found: {config_path}")
    
    try:
        with open(config_path, 'rb') as f:
            config = tomllib.load(f)
        return config
    except Exception as e:
        raise Exception(f"Failed to load virus configuration from {config_path}: {e}")


def get_virus_category(virus_name: str) -> str:
    """Get virus category from virus.toml configuration with caching."""
    global _virus_config_cache, _virus_category_cache
    
    # Return from cache if available
    if virus_name in _virus_category_cache:
        return _virus_category_cache[virus_name]
    
    # Load config if not cached
    if _virus_config_cache is None:
        _virus_config_cache = load_virus_config()
    
    virus_section = _virus_config_cache.get('virus', {})
    
    for virus_data in virus_section.values():
        if virus_data.get('name') == virus_name:
            category = virus_data.get('type', 'unknown')
            _virus_category_cache[virus_name] = category
            return category
    
    _virus_category_cache[virus_name] = 'unknown'
    return 'unknown'


def get_viruses_by_visibility(visibility: str) -> List[str]:
    """Get viruses by visibility level from virus.toml with caching."""
    global _virus_config_cache
    
    # Load config if not cached
    if _virus_config_cache is None:
        _virus_config_cache = load_virus_config()
    
    virus_section = _virus_config_cache.get('virus', {})
    
    viruses = []
    for virus_data in virus_section.values():
        virus_name = virus_data.get('name')
        virus_visibility = virus_data.get('visibility')
        
        if not virus_name:
            continue
            
        # Private includes everything, public only includes public viruses
        if visibility == 'private' or virus_visibility == 'public':
            viruses.append(virus_name)
    
    return viruses


def get_sites_by_visibility(visibility: str) -> List[str]:
    """Get sites by visibility level from sites.toml."""
    if visibility == 'public':
        return get_public_sites()
    else:
        return get_private_sites()


def sanitize_name(name: str) -> str:
    """Sanitize names for filenames."""
    return re.sub(r'[^A-Za-z0-9]+', '_', name)


# LLM Functions (extracted from generate_report_cards.py)
def prepare_analytics_data(city_name, category, virus_data, all_data, available_viruses):
    """Prepare comprehensive analytics for LLM prompt (matching public.R)."""
    if len(virus_data) == 0:
        return "", ""
    
    # Get latest week
    latest_week = virus_data['week_label'].max()
    
    # Filter to category viruses (using filtered available_viruses instead of VIRUS_LEVELS)
    category_viruses = [v for v in available_viruses if get_virus_category(v) == category and v in virus_data['virus'].values]
    
    analytics_lines = []
    virus_links = []  # Markdown links for LLM prompt
    virus_links_html = []  # HTML links for footer
    virus_link_map_lines = []
    
    for virus in category_viruses:
        virus_data_subset = virus_data[virus_data['virus'] == virus].copy()
        
        if len(virus_data_subset) == 0:
            continue
            
        # Get latest week data
        latest_data = virus_data_subset[virus_data_subset['week_label'] == latest_week]
        latest_reads = latest_data['mapped_reads'].sum() if len(latest_data) > 0 else 0
        
        # Calculate averages (simplified - would need more sophisticated time series analysis)
        # Long-term average (all available data)
        long_term_avg = virus_data_subset['mapped_reads'].mean()
        
        # Previous 4 weeks average
        recent_weeks = sorted(virus_data_subset['week_label'].unique())[-5:-1]  # Exclude latest week
        if len(recent_weeks) > 0:
            recent_data = virus_data_subset[virus_data_subset['week_label'].isin(recent_weeks)]
            four_week_avg = recent_data['mapped_reads'].mean()
        else:
            four_week_avg = 0
        
        # Seasonal average (same weeks in previous years - simplified)
        seasonal_avg = long_term_avg  # Placeholder
        
        # Check if detected this week but absent in previous 4 weeks
        recent_weeks_data = virus_data_subset[virus_data_subset['week_label'].isin(recent_weeks)]
        newly_detected = latest_reads > 0 and recent_weeks_data['mapped_reads'].sum() == 0
        
        # Build analytics line
        analytics_line = f"{virus}: latest={latest_reads:.0f}, long_term_avg={long_term_avg:.1f}, seasonal_avg={seasonal_avg:.1f}, 4wk_avg={four_week_avg:.1f}"
        if newly_detected:
            analytics_line += ", newly_detected=TRUE"
        if latest_reads < 20 and latest_reads > 0:
            analytics_line += ", low_confidence=TRUE"
            
        analytics_lines.append(analytics_line)
        
        # Create virus links for heatmaps
        safe_virus_name = sanitize_name(virus)
        virus_link = f"[{virus}](../by_virus/{safe_virus_name}.html)"  # Markdown for LLM prompt
        virus_link_html = f'<a href="../by_virus/{safe_virus_name}.html">{virus}</a>'  # HTML for footer
        virus_links.append(virus_link)
        virus_links_html.append(virus_link_html)
        virus_link_map_lines.append(f"{virus}: ../by_virus/{safe_virus_name}.html")
    
    analytics_txt = "\n".join(analytics_lines)
    virus_link_map = "\n".join(virus_link_map_lines)
    
    return analytics_txt, virus_link_map, virus_links, virus_links_html, latest_week


def get_category_specific_instructions(category):
    """Get category-specific instructions for LLM prompts."""
    if category == 'respiratory':
        return "Always include sections on SARS-CoV-2, influenza A, influenza B, and the RSVs, even if unchanged."
    elif category == 'enteric':
        return "Focus only on enteric (gut) pathogens. Always include sections on noroviruses and rotaviruses as the primary pathogens of concern. Do not discuss respiratory viruses like SARS-CoV-2, influenza, or RSV."
    elif category == 'animal':
        return "Focus only on animal pathogens. These are viruses primarily affecting animals (livestock, pets, wildlife) that may be present in wastewater due to agricultural runoff, pet waste, or wildlife. Include sections on the most prevalent animal viruses detected. Do not discuss human respiratory or enteric viruses."
    else:
        return "Focus on the pathogens detected in the data."


def build_llm_prompt(city_name, category, analytics_txt, virus_link_map, latest_week):
    """Build the detailed LLM prompt matching public.R style."""
    prompt = f"""LOCATION: {city_name}
MOST RECENT WEEK: {latest_week}
ANALYTICS:
{analytics_txt}

Begin with a one paragraph summary of the latest week with data, comparing it to the long-term average, current seasonal average, and previous 4-week average. Use qualitative language; avoid exact numbers or percentages since wastewater reads are highly variable.
Write a Lungfish viral surveillance discussion in National Weather Service style.
Focus on pathogen data from the latest week where data is available.
Note that the latest week is almost never the most recent calendar week, so discussions should specifically reference the latest week where there is data throughout the discussions.
Do not use terms like "this week" or "last week" without specifying the date. Instead use the terms like "the most recent week with data" or "the latest week with data".
Compare the latest week to the long-term average, the current seasonal average,
and the previous 4-week average. Use qualitative language; avoid exact numbers or
percentages since wastewater reads are highly variable.
Highlight any pathogen detected this week but absent in the previous four weeks.
If the latest mapped-read count for a pathogen is below 20, note that levels are
near the assay's limit of detection and confidence is lower.
{get_category_specific_instructions(category)}
Put pathogens likely to cause more severe disease first.
For each pathogen discuss transmission routes, symptoms, seasonal patterns,
and available counter-measures. Mention licensed vaccines where applicable.
Do NOT speculate on broader public-health meaning beyond what is seen in wastewater.
Conclude with a section called "Summary and Implications" that has actionable community guidance.
Do not use the term "Community Guidance" or "Public Health Guidance" in the summary.
Use HTML formatting with <b> tags for bold text and <a href> tags for links. No markdown formatting.

Use these exact sanitized heat-map URLs (match filename to virus):
{virus_link_map}

And when you hyperlink, link the virus name itself to its URL using <a href="URL">virus name</a>.
Link to virus names themselves, not a generic word like 'heatmap'.
At the start of each virus's discussion, include the virus name in <b>bold</b> tags.
Do not include the location or date in the discussion since I add them separately in a header."""

    return prompt


def get_forecast_from_llm(prompt: str, config: AzureOpenAIConfig) -> str:
    """Get forecast from Azure OpenAI API."""
    if not config.is_configured():
        console = Console()
        console.print("[red]Error: AZURE_OPENAI_API_KEY not found in environment[/red]")
        return "Error: Azure OpenAI API key not configured"
    
    # Construct the Azure OpenAI API URL
    url = f"{config.endpoint}/openai/deployments/{config.deployment}/chat/completions?api-version={config.api_version}"
    
    headers = {
        "Content-Type": "application/json",
        "api-key": config.api_key
    }
    
    # Prepare the request payload
    payload = {
        "messages": [
            {
                "role": "system",
                "content": "You are a public health expert specializing in wastewater viral surveillance. Write detailed, accurate, and professional viral forecast discussions in National Weather Service style. Use HTML formatting with <b> tags for bold text and <a href> tags for links. Do not use markdown formatting."
            },
            {
                "role": "user", 
                "content": prompt
            }
        ],
        "max_tokens": 2000,
        "temperature": 0.3,
        "top_p": 0.9,
        "frequency_penalty": 0,
        "presence_penalty": 0
    }
    
    try:
        console = Console()
        console.print("[blue]Making API call to Azure OpenAI...[/blue]")
        
        response = requests.post(url, headers=headers, json=payload, timeout=60)
        response.raise_for_status()
        
        result = response.json()
        
        if 'choices' in result and len(result['choices']) > 0:
            forecast_text = result['choices'][0]['message']['content'].strip()
            console.print("[green]✓ Successfully generated LLM discussion[/green]")
            return forecast_text
        else:
            console.print(f"[red]Unexpected API response format: {result}[/red]")
            return "Error: Unexpected API response format"
            
    except requests.exceptions.RequestException as e:
        console = Console()
        console.print(f"[red]Azure OpenAI API request failed: {e}[/red]")
        return f"Error: API request failed - {str(e)}"
    except Exception as e:
        console = Console()
        console.print(f"[red]Unexpected error calling Azure OpenAI: {e}[/red]")
        return f"Error: Unexpected error - {str(e)}"


def generate_llm_discussion(city_name, category, virus_data, all_data, available_viruses, config: AzureOpenAIConfig):
    """Generate comprehensive LLM-based discussion for city virus patterns."""
    if not config.is_configured():
        return None
    
    # Prepare analytics data
    analytics_txt, virus_link_map, virus_links, virus_links_html, latest_week = prepare_analytics_data(
        city_name, category, virus_data, all_data, available_viruses
    )
    
    if not analytics_txt:
        return None
    
    # Build prompt
    prompt = build_llm_prompt(city_name, category, analytics_txt, virus_link_map, latest_week)
    
    # Get forecast from LLM
    forecast_content = get_forecast_from_llm(prompt, config)
    
    # Build heading and footing
    azure_deployment = config.deployment or "GPT-4O"
    report_date = datetime.now().strftime('%Y-%m-%d')
    
    heading = f"""LUNGFISH VIRAL WASTEWATER DISCUSSION - PRE-ALPHA
{city_name.upper()}
{report_date}
LATEST DATA: {latest_week}
PREPARED BY {azure_deployment.upper()}

"""
    
    footing = f"""

These viruses are included in this discussion. {', '.join(virus_links_html)} are included. The methodology for the analysis is available here <a href="https://docs.google.com/document/d/1EmvCnpLgZYycb_i1VpaYDW5mWONHzudyzTvw0SHXdnw/edit?usp=sharing">link</a>.

DISCUSSION IS INFORMATIONAL AND SHOULD NOT BE USED FOR PUBLIC HEALTH OR MEDICAL DECISION MAKING
"""
    
    full_discussion = heading + forecast_content + footing
    
    return {
        'city': city_name,
        'category': category,
        'latest_week': latest_week,
        'prompt': prompt,
        'forecast': forecast_content,
        'full_discussion': full_discussion,
        'virus_links': virus_links,
        'virus_links_html': virus_links_html,
        'generated_at': datetime.now().isoformat()
    }


def save_llm_discussion(discussions_dir: Path, city_name: str, category: str, discussion: Dict) -> None:
    """Save LLM discussion to both JSON and HTML files."""
    if discussion is None:
        return None
    
    discussions_dir.mkdir(exist_ok=True)
    
    safe_city_name = sanitize_name(city_name)
    
    # Save JSON for debugging/archival
    json_file = discussions_dir / f'{safe_city_name}_{category}_discussion.json'
    with open(json_file, 'w', encoding='utf-8') as f:
        json.dump(discussion, f, indent=2, ensure_ascii=False)
    
    # Save HTML for linking from heatmaps
    html_file = discussions_dir / f'{safe_city_name}_{category}_discussion.html'
    
    html_content = f"""<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <title>Viral Discussion - {city_name} ({category.title()})</title>
    <style>
        body {{ font-family: monospace; margin: 40px; line-height: 1.6; max-width: 800px; }}
        h1 {{ color: #333; font-size: 18px; }}
        .header {{ background-color: #f5f5f5; padding: 15px; margin-bottom: 20px; white-space: pre-line; }}
        .content {{ margin: 20px 0; white-space: pre-line; }}
        .footer {{ background-color: #fff3cd; padding: 15px; margin-top: 20px; white-space: pre-line; border: 1px solid #ffeaa7; }}
        a {{ color: #0066cc; text-decoration: none; }}
        a:hover {{ text-decoration: underline; }}
    </style>
</head>
<body>
    <h1>Viral Wastewater Discussion - {city_name} ({category.title()})</h1>
    
    <div class="header">{discussion['full_discussion'].split(discussion['forecast'])[0]}</div>
    
    <div class="content">{discussion['forecast']}</div>
    
    <div class="footer">{discussion['full_discussion'].split(discussion['forecast'])[-1]}</div>
    
    <hr>
    <p><small>Generated: {discussion['generated_at']}</small></p>
</body>
</html>"""
    
    with open(html_file, 'w', encoding='utf-8') as f:
        f.write(html_content)
    
    return json_file, html_file


# Data Loading and Processing Functions
def prepare_data_for_discussions(df: pd.DataFrame, visibility: str) -> tuple:
    """
    Prepare report card data for discussion generation by filtering and aggregating.
    Reuses logic from create_visualizations.py.
    """
    console = Console()
    
    if df.empty:
        return df, set()
    
    # Filter viruses by visibility
    allowed_viruses = get_viruses_by_visibility(visibility)
    virus_filtered_df = df[df['virus'].isin(allowed_viruses)]
    
    # Filter sites by visibility  
    allowed_sites = get_sites_by_visibility(visibility)
    site_filtered_df = virus_filtered_df[virus_filtered_df['sewershed_location'].isin(allowed_sites)]
    
    console.print(f"[blue]Data filtering for {visibility} discussions:[/blue]")
    console.print(f"  Original records: {len(df)}")
    console.print(f"  After virus filtering: {len(virus_filtered_df)} (kept {len(allowed_viruses)} viruses)")
    console.print(f"  After site filtering: {len(site_filtered_df)} (kept {len(allowed_sites)} sites)")
    
    # Calculate reads per billion if not already present
    if 'reads_per_billion' not in site_filtered_df.columns:
        site_filtered_df = site_filtered_df.copy()
        site_filtered_df['reads_per_billion'] = 0.0
        mask = (site_filtered_df['total_reads'] > 0) & (site_filtered_df['mapped_reads'] >= 0)
        site_filtered_df.loc[mask, 'reads_per_billion'] = (
            site_filtered_df.loc[mask, 'mapped_reads'].astype(float) / 
            site_filtered_df.loc[mask, 'total_reads'].astype(float)
        ) * 1e9
    
    # Convert time_label to datetime and add week aggregation
    site_filtered_df['sample_date'] = pd.to_datetime(site_filtered_df['time_label'], errors='coerce')
    
    # Add week aggregation exactly like create_visualizations.py
    if site_filtered_df['sample_date'].notna().any():
        site_filtered_df['week_start'] = site_filtered_df['sample_date'].dt.to_period('W-SAT').dt.start_time
        site_filtered_df['week_label'] = site_filtered_df['week_start'].dt.strftime('%Y-%m-%d')
        site_filtered_df['week_date'] = pd.to_datetime(site_filtered_df['week_label'], errors='coerce')
    else:
        console.print("[yellow]No valid sample dates found[/yellow]")
        return pd.DataFrame(), set()
    
    # Create sequencing coverage set
    sequencing_coverage = set()
    valid_coverage = site_filtered_df.dropna(subset=['sewershed_location', 'week_label'])
    sequencing_coverage = set(zip(valid_coverage['sewershed_location'], valid_coverage['week_label']))
    
    # Aggregate by week, location, and virus
    agg_cols = ['sewershed_location', 'virus', 'week_label']
    agg_dict = {
        'mapped_reads': 'sum',
        'total_reads': 'first',  # Take first total reads (represents sequencing depth)
        'reads_per_billion': 'sum',  # Sum the normalized values
        'sample_date': 'first',
        'week_start': 'first',
        'week_date': 'first'
    }
    
    agg_df = site_filtered_df.groupby(agg_cols).agg(agg_dict).reset_index()
    
    console.print(f"[green]✓ Aggregated to {len(agg_df)} records for discussion generation[/green]")
    
    return agg_df, sequencing_coverage


def copy_visualizations_to_output(source_vis_dir: Path, target_vis_dir: Path) -> None:
    """Copy visualization files from source to target directory."""
    if source_vis_dir.exists():
        shutil.copytree(source_vis_dir, target_vis_dir, dirs_exist_ok=True)
        console = Console()
        console.print(f"[green]✓ Copied visualizations from {source_vis_dir} to {target_vis_dir}[/green]")


def manage_visualization_outputs(environment: str, base_dir: Path) -> None:
    """Copy visualizations to appropriate output directories for GitHub Actions."""
    if environment == "github-actions":
        script_dir = Path(__file__).parent
        source_local = script_dir / "local-outputs"
        
        # Copy public and private visualizations
        if (source_local / "visualizations_public").exists():
            copy_visualizations_to_output(
                source_local / "visualizations_public",
                base_dir / "visualizations_public"
            )
        
        if (source_local / "visualizations_private").exists():
            copy_visualizations_to_output(
                source_local / "visualizations_private", 
                base_dir / "visualizations_private"
            )


def process_city_discussions(city_name: str, virus_data: pd.DataFrame, all_data: pd.DataFrame, 
                           available_viruses: List[str], discussions_private_dir: Path, 
                           config: AzureOpenAIConfig) -> Dict[str, Any]:
    """Process all virus categories for a single city, generating one comprehensive discussion per category."""
    categories = ['respiratory', 'enteric', 'animal', 'atypical']
    results = {}
    console = Console()
    
    # Filter virus_data for this city
    city_data = virus_data[virus_data['sewershed_location'] == city_name]
    
    if city_data.empty:
        console.print(f"[yellow]No data for {city_name}, skipping discussions[/yellow]")
        return results
    
    console.print(f"[cyan]Processing discussions for {city_name}...[/cyan]")
    
    for category in categories:
        # Check if this category has any viruses for this city
        category_viruses = [v for v in available_viruses if get_virus_category(v) == category and v in city_data['virus'].values]
        
        if not category_viruses:
            console.print(f"[yellow]  No {category} viruses found for {city_name}, skipping[/yellow]")
            continue
        
        console.print(f"[blue]  Generating {category} discussion for {city_name}...[/blue]")
        
        try:
            # Generate discussion using existing functions with all private viruses
            discussion = generate_llm_discussion(
                city_name, category, city_data, all_data, available_viruses, config
            )
            
            if discussion:
                # Save discussion files to private directory
                json_file, html_file = save_llm_discussion(discussions_private_dir, city_name, category, discussion)
                results[f"{city_name}_{category}"] = {
                    'success': True,
                    'json_file': json_file,
                    'html_file': html_file,
                    'latest_week': discussion['latest_week'],
                    'category': category
                }
                console.print(f"[green]  ✓ Generated {category} discussion for {city_name}[/green]")
            else:
                console.print(f"[yellow]  No discussion generated for {city_name} {category}[/yellow]")
                results[f"{city_name}_{category}"] = {'success': False, 'reason': 'No analytics data'}
                
        except Exception as e:
            console.print(f"[red]  ✗ Error generating {category} discussion for {city_name}: {e}[/red]")
            results[f"{city_name}_{category}"] = {'success': False, 'reason': str(e)}
    
    return results


def copy_discussions_to_public(discussions_private_dir: Path, discussions_public_dir: Path, 
                              results: Dict[str, Any], public_sites: List[str]) -> None:
    """Copy discussions for public sites from private to public directory."""
    console = Console()
    console.print(f"[blue]Copying discussions for public sites...[/blue]")
    
    copied_count = 0
    for result_key, result_data in results.items():
        if not result_data.get('success', False):
            continue
            
        # Extract city name from result key
        city_name = result_key.rsplit('_', 1)[0]  # Remove category suffix
        
        # Check if this city should be in public
        if city_name in public_sites:
            json_file = result_data['json_file']
            html_file = result_data['html_file']
            
            # Copy files to public directory
            if json_file and json_file.exists():
                public_json = discussions_public_dir / json_file.name
                shutil.copy2(json_file, public_json)
            
            if html_file and html_file.exists():
                public_html = discussions_public_dir / html_file.name
                shutil.copy2(html_file, public_html)
                
            copied_count += 1
            console.print(f"[green]  ✓ Copied {result_key} to public[/green]")
    
    console.print(f"[green]✓ Copied {copied_count} discussions to public directory[/green]")


def load_cached_data(cache_dir: Path, visibility: str, use_cache: bool = True) -> Optional[pd.DataFrame]:
    """Load cached report card data if available."""
    console = Console()
    
    if not use_cache:
        return None
    
    # Look for the most recent cache file
    cache_pattern = f'report_card_data_all_*.pkl.gz'
    cache_files = list(cache_dir.glob(cache_pattern))
    
    if not cache_files:
        console.print(f"[yellow]No cached report card data found in {cache_dir}[/yellow]")
        return None
    
    # Get the most recent cache file
    latest_cache = max(cache_files, key=lambda p: p.stat().st_mtime)
    
    try:
        console.print(f"[cyan]Loading cached report card data: {latest_cache}[/cyan]")
        
        with gzip.open(latest_cache, 'rb') as f:
            cache_data = pickle.load(f)
        
        data = cache_data['data']
        cache_timestamp = cache_data['timestamp']
        age_hours = (datetime.now() - cache_timestamp).total_seconds() / 3600
        
        console.print(f"[green]✓ Loaded {len(data)} report card records from cache[/green]")
        console.print(f"[blue]Cache age: {age_hours:.1f} hours[/blue]")
        
        return data
        
    except Exception as e:
        console.print(f"[red]Error loading cached data: {e}[/red]")
        return None


def create_discussions(test_subset: bool = False,
                      max_cities: Optional[int] = None,
                      use_cache: bool = True) -> None:
    """Main function to create all discussions using comprehensive private dataset."""
    
    console = Console()
    
    # 1. Environment detection
    environment = detect_environment()
    console.print(f"[blue]Running in {environment} environment[/blue]")
    
    # 2. Setup output directories
    base_dir = get_output_base_dir(environment)
    output_dirs = setup_output_directories(base_dir)
    
    console.print(f"[blue]Output directories:[/blue]")
    for name, path in output_dirs.items():
        console.print(f"  {name}: {path}")
    
    # 3. Configure LLM
    config = AzureOpenAIConfig()
    if not config.is_configured():
        console.print("[red]Error: Azure OpenAI not configured[/red]")
        console.print("Set AZURE_OPENAI_API_KEY environment variable")
        sys.exit(1)
    
    console.print(f"[green]✓ Azure OpenAI configured (endpoint: {config.endpoint})[/green]")
    
    # 4. Load data
    console.print(f"[blue]Loading data for discussion generation...[/blue]")
    
    # Try to load cached data first
    # Use environment-aware cache directory
    cache_dir = base_dir / 'cache'
    report_data = load_cached_data(cache_dir, 'private', use_cache)
    
    if report_data is None:
        console.print("[red]Error: No cached data available[/red]")
        console.print("Run create_visualizations.py first to generate cached data")
        sys.exit(1)
    
    # 5. Generate comprehensive discussions using private data (includes all viruses and sites)
    console.print(f"\n[bold blue]Generating comprehensive discussions using private dataset...[/bold blue]")
    
    # Always use private data for comprehensive discussions
    discussion_data, sequencing_coverage = prepare_data_for_discussions(report_data, 'private')
    
    if discussion_data.empty:
        console.print(f"[red]Error: No data available for discussion generation[/red]")
        sys.exit(1)
    
    # Get all cities and viruses from private dataset
    all_cities = get_sites_by_visibility('private')
    all_viruses = get_viruses_by_visibility('private')
    
    # Apply max_cities limit if specified
    if max_cities and max_cities < len(all_cities):
        all_cities = all_cities[:max_cities]
        console.print(f"[yellow]Limiting to first {max_cities} cities for testing[/yellow]")
    
    console.print(f"Processing {len(all_cities)} cities with {len(all_viruses)} viruses")
    
    # Process each city with comprehensive private data
    all_results = {}
    
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        console=console
    ) as progress:
        
        task = progress.add_task("Generating comprehensive discussions...", total=len(all_cities))
        
        for city in all_cities:
            city_results = process_city_discussions(
                city, discussion_data, discussion_data, all_viruses, 
                output_dirs['discussions_private'], config
            )
            all_results.update(city_results)
            progress.update(task, advance=1)
    
    # 6. Copy discussions for public sites to public directory
    public_sites = get_sites_by_visibility('public')
    copy_discussions_to_public(
        output_dirs['discussions_private'], 
        output_dirs['discussions_public'], 
        all_results, 
        public_sites
    )
    
    # 7. Copy visualizations if in GitHub Actions
    if environment == "github-actions":
        console.print(f"\n[blue]Copying visualizations for GitHub Actions...[/blue]")
        manage_visualization_outputs(environment, base_dir)
    
    # 8. Summary
    console.print(f"\n[bold green]✅ Discussion generation complete![/bold green]")
    
    successful = sum(1 for r in all_results.values() if r.get('success', False))
    total = len(all_results)
    console.print(f"[blue]Generated: {successful}/{total} discussions successfully[/blue]")
    console.print(f"[blue]Private discussions saved to: {output_dirs['discussions_private']}[/blue]")
    console.print(f"[blue]Public discussions (copies) saved to: {output_dirs['discussions_public']}[/blue]")
    
    # Count public copies
    public_count = sum(1 for r in all_results.values() 
                      if r.get('success', False) and 
                      r.get('json_file', Path()).parent.parent.name.endswith('public'))
    if public_count == 0:  # Count will be 0 because we copy after, so count manually
        public_sites = get_sites_by_visibility('public')
        public_copies = sum(1 for result_key, result_data in all_results.items() 
                           if result_data.get('success', False) and 
                           result_key.rsplit('_', 1)[0] in public_sites)
        console.print(f"[blue]Public site discussions copied: {public_copies}[/blue]")


def parse_args():
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(
        description='Generate AI-powered virus surveillance discussions',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  # Generate comprehensive discussions for all sites
  python create_discussions.py
  
  # Test with limited cities
  python create_discussions.py --max-cities 3
  
  # Use cached data (default)
  python create_discussions.py --use-cache
  
  # Force fresh data generation (requires running create_visualizations.py first)
  python create_discussions.py --no-cache
        """
    )
    
    parser.add_argument('--test-subset', action='store_true',
                       help='Generate subset for testing')
    parser.add_argument('--max-cities', type=int, 
                       help='Limit number of cities (for testing)')
    parser.add_argument('--use-cache', action='store_true', default=True,
                       help='Use cached data (default)')
    parser.add_argument('--no-cache', action='store_true',
                       help='Force fresh data generation')
    
    return parser.parse_args()


if __name__ == "__main__":
    args = parse_args()
    
    # Handle cache flags
    use_cache = args.use_cache and not args.no_cache
    
    create_discussions(
        test_subset=args.test_subset,
        max_cities=args.max_cities,
        use_cache=use_cache
    )